// This file is generated

// Copyright (c) 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "content/browser/devtools/protocol/tracing.h"

#include "content/browser/devtools/protocol/protocol.h"

namespace content {
namespace protocol {
namespace Tracing {

// ------------- Enum values from types.

const char Metainfo::domainName[] = "Tracing";
const char Metainfo::commandPrefix[] = "Tracing.";
const char Metainfo::version[] = "1.2";

const char* TraceConfig::RecordModeEnum::RecordUntilFull = "recordUntilFull";
const char* TraceConfig::RecordModeEnum::RecordContinuously = "recordContinuously";
const char* TraceConfig::RecordModeEnum::RecordAsMuchAsPossible = "recordAsMuchAsPossible";
const char* TraceConfig::RecordModeEnum::EchoToConsole = "echoToConsole";

std::unique_ptr<TraceConfig> TraceConfig::parse(protocol::Value* value, ErrorSupport* errors)
{
    if (!value || value->type() != protocol::Value::TypeObject) {
        errors->addError("object expected");
        return nullptr;
    }

    std::unique_ptr<TraceConfig> result(new TraceConfig());
    protocol::DictionaryValue* object = DictionaryValue::cast(value);
    errors->push();
    protocol::Value* recordModeValue = object->get("recordMode");
    if (recordModeValue) {
        errors->setName("recordMode");
        result->m_recordMode = ValueConversions<String>::parse(recordModeValue, errors);
    }
    protocol::Value* enableSamplingValue = object->get("enableSampling");
    if (enableSamplingValue) {
        errors->setName("enableSampling");
        result->m_enableSampling = ValueConversions<bool>::parse(enableSamplingValue, errors);
    }
    protocol::Value* enableSystraceValue = object->get("enableSystrace");
    if (enableSystraceValue) {
        errors->setName("enableSystrace");
        result->m_enableSystrace = ValueConversions<bool>::parse(enableSystraceValue, errors);
    }
    protocol::Value* enableArgumentFilterValue = object->get("enableArgumentFilter");
    if (enableArgumentFilterValue) {
        errors->setName("enableArgumentFilter");
        result->m_enableArgumentFilter = ValueConversions<bool>::parse(enableArgumentFilterValue, errors);
    }
    protocol::Value* includedCategoriesValue = object->get("includedCategories");
    if (includedCategoriesValue) {
        errors->setName("includedCategories");
        result->m_includedCategories = ValueConversions<protocol::Array<String>>::parse(includedCategoriesValue, errors);
    }
    protocol::Value* excludedCategoriesValue = object->get("excludedCategories");
    if (excludedCategoriesValue) {
        errors->setName("excludedCategories");
        result->m_excludedCategories = ValueConversions<protocol::Array<String>>::parse(excludedCategoriesValue, errors);
    }
    protocol::Value* syntheticDelaysValue = object->get("syntheticDelays");
    if (syntheticDelaysValue) {
        errors->setName("syntheticDelays");
        result->m_syntheticDelays = ValueConversions<protocol::Array<String>>::parse(syntheticDelaysValue, errors);
    }
    protocol::Value* memoryDumpConfigValue = object->get("memoryDumpConfig");
    if (memoryDumpConfigValue) {
        errors->setName("memoryDumpConfig");
        result->m_memoryDumpConfig = ValueConversions<protocol::Tracing::MemoryDumpConfig>::parse(memoryDumpConfigValue, errors);
    }
    errors->pop();
    if (errors->hasErrors())
        return nullptr;
    return result;
}

std::unique_ptr<protocol::DictionaryValue> TraceConfig::serialize() const
{
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    if (m_recordMode.isJust())
        result->setValue("recordMode", ValueConversions<String>::serialize(m_recordMode.fromJust()));
    if (m_enableSampling.isJust())
        result->setValue("enableSampling", ValueConversions<bool>::serialize(m_enableSampling.fromJust()));
    if (m_enableSystrace.isJust())
        result->setValue("enableSystrace", ValueConversions<bool>::serialize(m_enableSystrace.fromJust()));
    if (m_enableArgumentFilter.isJust())
        result->setValue("enableArgumentFilter", ValueConversions<bool>::serialize(m_enableArgumentFilter.fromJust()));
    if (m_includedCategories.isJust())
        result->setValue("includedCategories", ValueConversions<protocol::Array<String>>::serialize(m_includedCategories.fromJust()));
    if (m_excludedCategories.isJust())
        result->setValue("excludedCategories", ValueConversions<protocol::Array<String>>::serialize(m_excludedCategories.fromJust()));
    if (m_syntheticDelays.isJust())
        result->setValue("syntheticDelays", ValueConversions<protocol::Array<String>>::serialize(m_syntheticDelays.fromJust()));
    if (m_memoryDumpConfig.isJust())
        result->setValue("memoryDumpConfig", ValueConversions<protocol::Tracing::MemoryDumpConfig>::serialize(m_memoryDumpConfig.fromJust()));
    return result;
}

std::unique_ptr<TraceConfig> TraceConfig::clone() const
{
    ErrorSupport errors;
    return parse(serialize().get(), &errors);
}

// ------------- Enum values from params.


namespace Start {
namespace TransferModeEnum {
const char* ReportEvents = "ReportEvents";
const char* ReturnAsStream = "ReturnAsStream";
} // namespace TransferModeEnum
} // namespace Start

// ------------- Frontend notifications.

void Frontend::DataCollected(std::unique_ptr<protocol::Array<protocol::DictionaryValue>> value)
{
    std::unique_ptr<protocol::DictionaryValue> jsonMessage = DictionaryValue::create();
    jsonMessage->setString("method", "Tracing.dataCollected");
    std::unique_ptr<protocol::DictionaryValue> paramsObject = DictionaryValue::create();
    paramsObject->setValue("value", ValueConversions<protocol::Array<protocol::DictionaryValue>>::serialize(value.get()));
    jsonMessage->setObject("params", std::move(paramsObject));
    if (m_frontendChannel)
        m_frontendChannel->sendProtocolNotification(jsonMessage->toJSONString());
}

void Frontend::TracingComplete(Maybe<String> stream)
{
    std::unique_ptr<protocol::DictionaryValue> jsonMessage = DictionaryValue::create();
    jsonMessage->setString("method", "Tracing.tracingComplete");
    std::unique_ptr<protocol::DictionaryValue> paramsObject = DictionaryValue::create();
    if (stream.isJust())
        paramsObject->setValue("stream", ValueConversions<String>::serialize(stream.fromJust()));
    jsonMessage->setObject("params", std::move(paramsObject));
    if (m_frontendChannel)
        m_frontendChannel->sendProtocolNotification(jsonMessage->toJSONString());
}

void Frontend::BufferUsage(Maybe<double> percentFull, Maybe<double> eventCount, Maybe<double> value)
{
    std::unique_ptr<protocol::DictionaryValue> jsonMessage = DictionaryValue::create();
    jsonMessage->setString("method", "Tracing.bufferUsage");
    std::unique_ptr<protocol::DictionaryValue> paramsObject = DictionaryValue::create();
    if (percentFull.isJust())
        paramsObject->setValue("percentFull", ValueConversions<double>::serialize(percentFull.fromJust()));
    if (eventCount.isJust())
        paramsObject->setValue("eventCount", ValueConversions<double>::serialize(eventCount.fromJust()));
    if (value.isJust())
        paramsObject->setValue("value", ValueConversions<double>::serialize(value.fromJust()));
    jsonMessage->setObject("params", std::move(paramsObject));
    if (m_frontendChannel)
        m_frontendChannel->sendProtocolNotification(jsonMessage->toJSONString());
}

void Frontend::flush()
{
    m_frontendChannel->flushProtocolNotifications();
}

// --------------------- Dispatcher.

class DispatcherImpl : public protocol::DispatcherBase {
public:
    DispatcherImpl(FrontendChannel* frontendChannel, Backend* backend)
        : DispatcherBase(frontendChannel)
        , m_backend(backend) {
        m_dispatchMap["Tracing.start"] = &DispatcherImpl::start;
        m_dispatchMap["Tracing.end"] = &DispatcherImpl::end;
        m_dispatchMap["Tracing.getCategories"] = &DispatcherImpl::getCategories;
        m_dispatchMap["Tracing.requestMemoryDump"] = &DispatcherImpl::requestMemoryDump;
        m_dispatchMap["Tracing.recordClockSyncMarker"] = &DispatcherImpl::recordClockSyncMarker;
    }
    ~DispatcherImpl() override { }
    DispatchResponse::Status dispatch(int callId, const String& method, std::unique_ptr<protocol::DictionaryValue> messageObject) override;

protected:
    using CallHandler = DispatchResponse::Status (DispatcherImpl::*)(int callId, std::unique_ptr<DictionaryValue> messageObject, ErrorSupport* errors);
    using DispatchMap = protocol::HashMap<String, CallHandler>;
    DispatchMap m_dispatchMap;

    DispatchResponse::Status start(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    DispatchResponse::Status end(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    DispatchResponse::Status getCategories(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    DispatchResponse::Status requestMemoryDump(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    DispatchResponse::Status recordClockSyncMarker(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);

    Backend* m_backend;
};

DispatchResponse::Status DispatcherImpl::dispatch(int callId, const String& method, std::unique_ptr<protocol::DictionaryValue> messageObject)
{
    protocol::HashMap<String, CallHandler>::iterator it = m_dispatchMap.find(method);
    if (it == m_dispatchMap.end()) {
        reportProtocolError(callId, DispatchResponse::kMethodNotFound, "'" + method + "' wasn't found", nullptr);
        return DispatchResponse::kError;
    }

    protocol::ErrorSupport errors;
    return (this->*(it->second))(callId, std::move(messageObject), &errors);
}


class StartCallbackImpl : public Backend::StartCallback, public DispatcherBase::Callback {
public:
    StartCallbackImpl(std::unique_ptr<DispatcherBase::WeakPtr> backendImpl, int callId)
        : DispatcherBase::Callback(std::move(backendImpl), callId) { }

    void sendSuccess() override
    {
        std::unique_ptr<protocol::DictionaryValue> resultObject = DictionaryValue::create();
        sendIfActive(std::move(resultObject), DispatchResponse::OK());
    }

    void sendFailure(const DispatchResponse& response) override
    {
        DCHECK(response.status() == DispatchResponse::kError);
        sendIfActive(nullptr, response);
    }
};

DispatchResponse::Status DispatcherImpl::start(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{
    // Prepare input parameters.
    protocol::DictionaryValue* object = DictionaryValue::cast(requestMessageObject->get("params"));
    errors->push();
    protocol::Value* categoriesValue = object ? object->get("categories") : nullptr;
    Maybe<String> in_categories;
    if (categoriesValue) {
        errors->setName("categories");
        in_categories = ValueConversions<String>::parse(categoriesValue, errors);
    }
    protocol::Value* optionsValue = object ? object->get("options") : nullptr;
    Maybe<String> in_options;
    if (optionsValue) {
        errors->setName("options");
        in_options = ValueConversions<String>::parse(optionsValue, errors);
    }
    protocol::Value* bufferUsageReportingIntervalValue = object ? object->get("bufferUsageReportingInterval") : nullptr;
    Maybe<double> in_bufferUsageReportingInterval;
    if (bufferUsageReportingIntervalValue) {
        errors->setName("bufferUsageReportingInterval");
        in_bufferUsageReportingInterval = ValueConversions<double>::parse(bufferUsageReportingIntervalValue, errors);
    }
    protocol::Value* transferModeValue = object ? object->get("transferMode") : nullptr;
    Maybe<String> in_transferMode;
    if (transferModeValue) {
        errors->setName("transferMode");
        in_transferMode = ValueConversions<String>::parse(transferModeValue, errors);
    }
    protocol::Value* traceConfigValue = object ? object->get("traceConfig") : nullptr;
    Maybe<protocol::Tracing::TraceConfig> in_traceConfig;
    if (traceConfigValue) {
        errors->setName("traceConfig");
        in_traceConfig = ValueConversions<protocol::Tracing::TraceConfig>::parse(traceConfigValue, errors);
    }
    errors->pop();
    if (errors->hasErrors()) {
        reportProtocolError(callId, DispatchResponse::kInvalidParams, kInvalidParamsString, errors);
        return DispatchResponse::kError;
    }

    std::unique_ptr<StartCallbackImpl> callback(new StartCallbackImpl(weakPtr(), callId));
    m_backend->Start(std::move(in_categories), std::move(in_options), std::move(in_bufferUsageReportingInterval), std::move(in_transferMode), std::move(in_traceConfig), std::move(callback));
    return DispatchResponse::kAsync;
}

class EndCallbackImpl : public Backend::EndCallback, public DispatcherBase::Callback {
public:
    EndCallbackImpl(std::unique_ptr<DispatcherBase::WeakPtr> backendImpl, int callId)
        : DispatcherBase::Callback(std::move(backendImpl), callId) { }

    void sendSuccess() override
    {
        std::unique_ptr<protocol::DictionaryValue> resultObject = DictionaryValue::create();
        sendIfActive(std::move(resultObject), DispatchResponse::OK());
    }

    void sendFailure(const DispatchResponse& response) override
    {
        DCHECK(response.status() == DispatchResponse::kError);
        sendIfActive(nullptr, response);
    }
};

DispatchResponse::Status DispatcherImpl::end(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{

    std::unique_ptr<EndCallbackImpl> callback(new EndCallbackImpl(weakPtr(), callId));
    m_backend->End(std::move(callback));
    return DispatchResponse::kAsync;
}

DispatchResponse::Status DispatcherImpl::getCategories(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{
    // Declare output parameters.
    std::unique_ptr<protocol::Array<String>> out_categories;

    std::unique_ptr<DispatcherBase::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->GetCategories(&out_categories);
    if (response.status() == DispatchResponse::kFallThrough)
        return response.status();
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    if (response.status() == DispatchResponse::kSuccess) {
        result->setValue("categories", ValueConversions<protocol::Array<String>>::serialize(out_categories.get()));
    }
    if (weak->get())
        weak->get()->sendResponse(callId, response, std::move(result));
    return response.status();
}

DispatchResponse::Status DispatcherImpl::requestMemoryDump(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{
    // Declare output parameters.
    String out_dumpGuid;
    bool out_success;

    std::unique_ptr<DispatcherBase::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->RequestMemoryDump(&out_dumpGuid, &out_success);
    if (response.status() == DispatchResponse::kFallThrough)
        return response.status();
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    if (response.status() == DispatchResponse::kSuccess) {
        result->setValue("dumpGuid", ValueConversions<String>::serialize(out_dumpGuid));
        result->setValue("success", ValueConversions<bool>::serialize(out_success));
    }
    if (weak->get())
        weak->get()->sendResponse(callId, response, std::move(result));
    return response.status();
}

DispatchResponse::Status DispatcherImpl::recordClockSyncMarker(int callId, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{
    // Prepare input parameters.
    protocol::DictionaryValue* object = DictionaryValue::cast(requestMessageObject->get("params"));
    errors->push();
    protocol::Value* syncIdValue = object ? object->get("syncId") : nullptr;
    errors->setName("syncId");
    String in_syncId = ValueConversions<String>::parse(syncIdValue, errors);
    errors->pop();
    if (errors->hasErrors()) {
        reportProtocolError(callId, DispatchResponse::kInvalidParams, kInvalidParamsString, errors);
        return DispatchResponse::kError;
    }

    std::unique_ptr<DispatcherBase::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->RecordClockSyncMarker(in_syncId);
    if (weak->get())
        weak->get()->sendResponse(callId, response);
    return response.status();
}

// static
void Dispatcher::wire(UberDispatcher* dispatcher, Backend* backend)
{
    dispatcher->registerBackend("Tracing", std::unique_ptr<protocol::DispatcherBase>(new DispatcherImpl(dispatcher->channel(), backend)));
}

} // Tracing
} // namespace content
} // namespace protocol
