// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_INDEXED_DB_INDEXED_DB_MOJOM_SHARED_INTERNAL_H_
#define CONTENT_COMMON_INDEXED_DB_INDEXED_DB_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/common/common_custom_types.mojom-shared-internal.h"
#include "url/mojo/origin.mojom-shared-internal.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace indexed_db {
namespace mojom {
namespace internal {
using Key_Data = mojo::internal::NativeStruct_Data;
using KeyPath_Data = mojo::internal::NativeStruct_Data;
using KeyRange_Data = mojo::internal::NativeStruct_Data;
class IndexMetadata_Data;
class ObjectStoreMetadata_Data;
class DatabaseMetadata_Data;
class IndexKeys_Data;
class FileInfo_Data;
class BlobInfo_Data;
class Value_Data;
class ReturnValue_Data;
using CursorDirection_Data =
    mojo::internal::NativeEnum_Data;
using DataLoss_Data =
    mojo::internal::NativeEnum_Data;
using PutMode_Data =
    mojo::internal::NativeEnum_Data;
using TaskType_Data =
    mojo::internal::NativeEnum_Data;
using TransactionMode_Data =
    mojo::internal::NativeEnum_Data;

#pragma pack(push, 1)
class IndexMetadata_Data {
 public:
  static IndexMetadata_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(IndexMetadata_Data))) IndexMetadata_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  mojo::internal::Pointer<internal::KeyPath_Data> key_path;
  uint8_t unique : 1;
  uint8_t multi_entry : 1;
  uint8_t padfinal_[7];

 private:
  IndexMetadata_Data() : header_({sizeof(*this), 0}) {
  }
  ~IndexMetadata_Data() = delete;
};
static_assert(sizeof(IndexMetadata_Data) == 40,
              "Bad sizeof(IndexMetadata_Data)");
class ObjectStoreMetadata_Data {
 public:
  static ObjectStoreMetadata_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(ObjectStoreMetadata_Data))) ObjectStoreMetadata_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  mojo::internal::Pointer<internal::KeyPath_Data> key_path;
  uint8_t auto_increment : 1;
  uint8_t pad3_[7];
  int64_t max_index_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::IndexMetadata_Data>>> indexes;

 private:
  ObjectStoreMetadata_Data() : header_({sizeof(*this), 0}) {
  }
  ~ObjectStoreMetadata_Data() = delete;
};
static_assert(sizeof(ObjectStoreMetadata_Data) == 56,
              "Bad sizeof(ObjectStoreMetadata_Data)");
class DatabaseMetadata_Data {
 public:
  static DatabaseMetadata_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DatabaseMetadata_Data))) DatabaseMetadata_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  int64_t version;
  int64_t max_object_store_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::ObjectStoreMetadata_Data>>> object_stores;

 private:
  DatabaseMetadata_Data() : header_({sizeof(*this), 0}) {
  }
  ~DatabaseMetadata_Data() = delete;
};
static_assert(sizeof(DatabaseMetadata_Data) == 48,
              "Bad sizeof(DatabaseMetadata_Data)");
class IndexKeys_Data {
 public:
  static IndexKeys_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(IndexKeys_Data))) IndexKeys_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t index_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::Key_Data>>> index_keys;

 private:
  IndexKeys_Data() : header_({sizeof(*this), 0}) {
  }
  ~IndexKeys_Data() = delete;
};
static_assert(sizeof(IndexKeys_Data) == 24,
              "Bad sizeof(IndexKeys_Data)");
class FileInfo_Data {
 public:
  static FileInfo_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(FileInfo_Data))) FileInfo_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::FilePath_Data> path;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  mojo::internal::Pointer<::mojo::common::mojom::internal::Time_Data> last_modified;

 private:
  FileInfo_Data() : header_({sizeof(*this), 0}) {
  }
  ~FileInfo_Data() = delete;
};
static_assert(sizeof(FileInfo_Data) == 32,
              "Bad sizeof(FileInfo_Data)");
class BlobInfo_Data {
 public:
  static BlobInfo_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(BlobInfo_Data))) BlobInfo_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> uuid;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> mime_type;
  uint64_t size;
  mojo::internal::Pointer<internal::FileInfo_Data> file;

 private:
  BlobInfo_Data() : header_({sizeof(*this), 0}) {
  }
  ~BlobInfo_Data() = delete;
};
static_assert(sizeof(BlobInfo_Data) == 40,
              "Bad sizeof(BlobInfo_Data)");
class Value_Data {
 public:
  static Value_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Value_Data))) Value_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> bits;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::BlobInfo_Data>>> blob_or_file_info;

 private:
  Value_Data() : header_({sizeof(*this), 0}) {
  }
  ~Value_Data() = delete;
};
static_assert(sizeof(Value_Data) == 24,
              "Bad sizeof(Value_Data)");
class ReturnValue_Data {
 public:
  static ReturnValue_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(ReturnValue_Data))) ReturnValue_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::Value_Data> value;
  mojo::internal::Pointer<internal::Key_Data> primary_key;
  mojo::internal::Pointer<internal::KeyPath_Data> key_path;

 private:
  ReturnValue_Data() : header_({sizeof(*this), 0}) {
  }
  ~ReturnValue_Data() = delete;
};
static_assert(sizeof(ReturnValue_Data) == 32,
              "Bad sizeof(ReturnValue_Data)");
const uint32_t kCallbacks_Error_Name = 0;
class Callbacks_Error_Params_Data {
 public:
  static Callbacks_Error_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_Error_Params_Data))) Callbacks_Error_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t code;
  uint8_t pad0_[4];
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> message;

 private:
  Callbacks_Error_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_Error_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_Error_Params_Data) == 24,
              "Bad sizeof(Callbacks_Error_Params_Data)");
const uint32_t kCallbacks_SuccessStringList_Name = 1;
class Callbacks_SuccessStringList_Params_Data {
 public:
  static Callbacks_SuccessStringList_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessStringList_Params_Data))) Callbacks_SuccessStringList_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data>>> value;

 private:
  Callbacks_SuccessStringList_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessStringList_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessStringList_Params_Data) == 16,
              "Bad sizeof(Callbacks_SuccessStringList_Params_Data)");
const uint32_t kCallbacks_Blocked_Name = 2;
class Callbacks_Blocked_Params_Data {
 public:
  static Callbacks_Blocked_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_Blocked_Params_Data))) Callbacks_Blocked_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t existing_version;

 private:
  Callbacks_Blocked_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_Blocked_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_Blocked_Params_Data) == 16,
              "Bad sizeof(Callbacks_Blocked_Params_Data)");
const uint32_t kCallbacks_UpgradeNeeded_Name = 3;
class Callbacks_UpgradeNeeded_Params_Data {
 public:
  static Callbacks_UpgradeNeeded_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_UpgradeNeeded_Params_Data))) Callbacks_UpgradeNeeded_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::AssociatedInterface_Data database;
  int64_t old_version;
  int32_t data_loss;
  uint8_t pad2_[4];
  mojo::internal::Pointer<mojo::internal::String_Data> data_loss_message;
  mojo::internal::Pointer<internal::DatabaseMetadata_Data> db_metadata;

 private:
  Callbacks_UpgradeNeeded_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_UpgradeNeeded_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_UpgradeNeeded_Params_Data) == 48,
              "Bad sizeof(Callbacks_UpgradeNeeded_Params_Data)");
const uint32_t kCallbacks_SuccessDatabase_Name = 4;
class Callbacks_SuccessDatabase_Params_Data {
 public:
  static Callbacks_SuccessDatabase_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessDatabase_Params_Data))) Callbacks_SuccessDatabase_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::AssociatedInterface_Data database;
  mojo::internal::Pointer<internal::DatabaseMetadata_Data> metadata;

 private:
  Callbacks_SuccessDatabase_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessDatabase_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessDatabase_Params_Data) == 24,
              "Bad sizeof(Callbacks_SuccessDatabase_Params_Data)");
const uint32_t kCallbacks_SuccessCursor_Name = 5;
class Callbacks_SuccessCursor_Params_Data {
 public:
  static Callbacks_SuccessCursor_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessCursor_Params_Data))) Callbacks_SuccessCursor_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t cursor_id;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::Key_Data> key;
  mojo::internal::Pointer<internal::Key_Data> primary_key;
  mojo::internal::Pointer<internal::Value_Data> value;

 private:
  Callbacks_SuccessCursor_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessCursor_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessCursor_Params_Data) == 40,
              "Bad sizeof(Callbacks_SuccessCursor_Params_Data)");
const uint32_t kCallbacks_SuccessValue_Name = 6;
class Callbacks_SuccessValue_Params_Data {
 public:
  static Callbacks_SuccessValue_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessValue_Params_Data))) Callbacks_SuccessValue_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::ReturnValue_Data> value;

 private:
  Callbacks_SuccessValue_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessValue_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessValue_Params_Data) == 16,
              "Bad sizeof(Callbacks_SuccessValue_Params_Data)");
const uint32_t kCallbacks_SuccessArray_Name = 7;
class Callbacks_SuccessArray_Params_Data {
 public:
  static Callbacks_SuccessArray_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessArray_Params_Data))) Callbacks_SuccessArray_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::ReturnValue_Data>>> values;

 private:
  Callbacks_SuccessArray_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessArray_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessArray_Params_Data) == 16,
              "Bad sizeof(Callbacks_SuccessArray_Params_Data)");
const uint32_t kCallbacks_SuccessKey_Name = 8;
class Callbacks_SuccessKey_Params_Data {
 public:
  static Callbacks_SuccessKey_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessKey_Params_Data))) Callbacks_SuccessKey_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::Key_Data> key;

 private:
  Callbacks_SuccessKey_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessKey_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessKey_Params_Data) == 16,
              "Bad sizeof(Callbacks_SuccessKey_Params_Data)");
const uint32_t kCallbacks_SuccessInteger_Name = 9;
class Callbacks_SuccessInteger_Params_Data {
 public:
  static Callbacks_SuccessInteger_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_SuccessInteger_Params_Data))) Callbacks_SuccessInteger_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t value;

 private:
  Callbacks_SuccessInteger_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_SuccessInteger_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_SuccessInteger_Params_Data) == 16,
              "Bad sizeof(Callbacks_SuccessInteger_Params_Data)");
const uint32_t kCallbacks_Success_Name = 10;
class Callbacks_Success_Params_Data {
 public:
  static Callbacks_Success_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Callbacks_Success_Params_Data))) Callbacks_Success_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  Callbacks_Success_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Callbacks_Success_Params_Data() = delete;
};
static_assert(sizeof(Callbacks_Success_Params_Data) == 8,
              "Bad sizeof(Callbacks_Success_Params_Data)");
const uint32_t kDatabaseCallbacks_ForcedClose_Name = 0;
class DatabaseCallbacks_ForcedClose_Params_Data {
 public:
  static DatabaseCallbacks_ForcedClose_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DatabaseCallbacks_ForcedClose_Params_Data))) DatabaseCallbacks_ForcedClose_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  DatabaseCallbacks_ForcedClose_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~DatabaseCallbacks_ForcedClose_Params_Data() = delete;
};
static_assert(sizeof(DatabaseCallbacks_ForcedClose_Params_Data) == 8,
              "Bad sizeof(DatabaseCallbacks_ForcedClose_Params_Data)");
const uint32_t kDatabaseCallbacks_VersionChange_Name = 1;
class DatabaseCallbacks_VersionChange_Params_Data {
 public:
  static DatabaseCallbacks_VersionChange_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DatabaseCallbacks_VersionChange_Params_Data))) DatabaseCallbacks_VersionChange_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t old_version;
  int64_t new_version;

 private:
  DatabaseCallbacks_VersionChange_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~DatabaseCallbacks_VersionChange_Params_Data() = delete;
};
static_assert(sizeof(DatabaseCallbacks_VersionChange_Params_Data) == 24,
              "Bad sizeof(DatabaseCallbacks_VersionChange_Params_Data)");
const uint32_t kDatabaseCallbacks_Abort_Name = 2;
class DatabaseCallbacks_Abort_Params_Data {
 public:
  static DatabaseCallbacks_Abort_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DatabaseCallbacks_Abort_Params_Data))) DatabaseCallbacks_Abort_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int32_t code;
  uint8_t pad1_[4];
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> message;

 private:
  DatabaseCallbacks_Abort_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~DatabaseCallbacks_Abort_Params_Data() = delete;
};
static_assert(sizeof(DatabaseCallbacks_Abort_Params_Data) == 32,
              "Bad sizeof(DatabaseCallbacks_Abort_Params_Data)");
const uint32_t kDatabaseCallbacks_Complete_Name = 3;
class DatabaseCallbacks_Complete_Params_Data {
 public:
  static DatabaseCallbacks_Complete_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DatabaseCallbacks_Complete_Params_Data))) DatabaseCallbacks_Complete_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;

 private:
  DatabaseCallbacks_Complete_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~DatabaseCallbacks_Complete_Params_Data() = delete;
};
static_assert(sizeof(DatabaseCallbacks_Complete_Params_Data) == 16,
              "Bad sizeof(DatabaseCallbacks_Complete_Params_Data)");
const uint32_t kDatabase_CreateObjectStore_Name = 0;
class Database_CreateObjectStore_Params_Data {
 public:
  static Database_CreateObjectStore_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_CreateObjectStore_Params_Data))) Database_CreateObjectStore_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  mojo::internal::Pointer<internal::KeyPath_Data> key_path;
  uint8_t auto_increment : 1;
  uint8_t padfinal_[7];

 private:
  Database_CreateObjectStore_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_CreateObjectStore_Params_Data() = delete;
};
static_assert(sizeof(Database_CreateObjectStore_Params_Data) == 48,
              "Bad sizeof(Database_CreateObjectStore_Params_Data)");
const uint32_t kDatabase_DeleteObjectStore_Name = 1;
class Database_DeleteObjectStore_Params_Data {
 public:
  static Database_DeleteObjectStore_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_DeleteObjectStore_Params_Data))) Database_DeleteObjectStore_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;

 private:
  Database_DeleteObjectStore_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_DeleteObjectStore_Params_Data() = delete;
};
static_assert(sizeof(Database_DeleteObjectStore_Params_Data) == 24,
              "Bad sizeof(Database_DeleteObjectStore_Params_Data)");
const uint32_t kDatabase_RenameObjectStore_Name = 2;
class Database_RenameObjectStore_Params_Data {
 public:
  static Database_RenameObjectStore_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_RenameObjectStore_Params_Data))) Database_RenameObjectStore_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> new_name;

 private:
  Database_RenameObjectStore_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_RenameObjectStore_Params_Data() = delete;
};
static_assert(sizeof(Database_RenameObjectStore_Params_Data) == 32,
              "Bad sizeof(Database_RenameObjectStore_Params_Data)");
const uint32_t kDatabase_CreateTransaction_Name = 3;
class Database_CreateTransaction_Params_Data {
 public:
  static Database_CreateTransaction_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_CreateTransaction_Params_Data))) Database_CreateTransaction_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<int64_t>> object_store_ids;
  int32_t mode;
  uint8_t padfinal_[4];

 private:
  Database_CreateTransaction_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_CreateTransaction_Params_Data() = delete;
};
static_assert(sizeof(Database_CreateTransaction_Params_Data) == 32,
              "Bad sizeof(Database_CreateTransaction_Params_Data)");
const uint32_t kDatabase_Close_Name = 4;
class Database_Close_Params_Data {
 public:
  static Database_Close_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Close_Params_Data))) Database_Close_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  Database_Close_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Close_Params_Data() = delete;
};
static_assert(sizeof(Database_Close_Params_Data) == 8,
              "Bad sizeof(Database_Close_Params_Data)");
const uint32_t kDatabase_VersionChangeIgnored_Name = 5;
class Database_VersionChangeIgnored_Params_Data {
 public:
  static Database_VersionChangeIgnored_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_VersionChangeIgnored_Params_Data))) Database_VersionChangeIgnored_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  Database_VersionChangeIgnored_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_VersionChangeIgnored_Params_Data() = delete;
};
static_assert(sizeof(Database_VersionChangeIgnored_Params_Data) == 8,
              "Bad sizeof(Database_VersionChangeIgnored_Params_Data)");
const uint32_t kDatabase_AddObserver_Name = 6;
class Database_AddObserver_Params_Data {
 public:
  static Database_AddObserver_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_AddObserver_Params_Data))) Database_AddObserver_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int32_t observer_id;
  uint8_t include_transaction : 1;
  uint8_t no_records : 1;
  uint8_t values : 1;
  uint8_t pad4_[1];
  uint16_t operation_types;

 private:
  Database_AddObserver_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_AddObserver_Params_Data() = delete;
};
static_assert(sizeof(Database_AddObserver_Params_Data) == 24,
              "Bad sizeof(Database_AddObserver_Params_Data)");
const uint32_t kDatabase_RemoveObservers_Name = 7;
class Database_RemoveObservers_Params_Data {
 public:
  static Database_RemoveObservers_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_RemoveObservers_Params_Data))) Database_RemoveObservers_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> observers;

 private:
  Database_RemoveObservers_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_RemoveObservers_Params_Data() = delete;
};
static_assert(sizeof(Database_RemoveObservers_Params_Data) == 16,
              "Bad sizeof(Database_RemoveObservers_Params_Data)");
const uint32_t kDatabase_Get_Name = 8;
class Database_Get_Params_Data {
 public:
  static Database_Get_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Get_Params_Data))) Database_Get_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<internal::KeyRange_Data> key_range;
  uint8_t key_only : 1;
  uint8_t pad4_[3];
  mojo::internal::AssociatedInterface_Data callbacks;
  uint8_t padfinal_[4];

 private:
  Database_Get_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Get_Params_Data() = delete;
};
static_assert(sizeof(Database_Get_Params_Data) == 56,
              "Bad sizeof(Database_Get_Params_Data)");
const uint32_t kDatabase_GetAll_Name = 9;
class Database_GetAll_Params_Data {
 public:
  static Database_GetAll_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_GetAll_Params_Data))) Database_GetAll_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<internal::KeyRange_Data> key_range;
  uint8_t key_only : 1;
  uint8_t pad4_[7];
  int64_t max_count;
  mojo::internal::AssociatedInterface_Data callbacks;

 private:
  Database_GetAll_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_GetAll_Params_Data() = delete;
};
static_assert(sizeof(Database_GetAll_Params_Data) == 64,
              "Bad sizeof(Database_GetAll_Params_Data)");
const uint32_t kDatabase_Put_Name = 10;
class Database_Put_Params_Data {
 public:
  static Database_Put_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Put_Params_Data))) Database_Put_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<internal::Value_Data> value;
  mojo::internal::Pointer<internal::Key_Data> key;
  int32_t mode;
  uint8_t pad4_[4];
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::IndexKeys_Data>>> index_keys;
  mojo::internal::AssociatedInterface_Data callbacks;

 private:
  Database_Put_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Put_Params_Data() = delete;
};
static_assert(sizeof(Database_Put_Params_Data) == 64,
              "Bad sizeof(Database_Put_Params_Data)");
const uint32_t kDatabase_SetIndexKeys_Name = 11;
class Database_SetIndexKeys_Params_Data {
 public:
  static Database_SetIndexKeys_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_SetIndexKeys_Params_Data))) Database_SetIndexKeys_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<internal::Key_Data> primary_key;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::IndexKeys_Data>>> index_keys;

 private:
  Database_SetIndexKeys_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_SetIndexKeys_Params_Data() = delete;
};
static_assert(sizeof(Database_SetIndexKeys_Params_Data) == 40,
              "Bad sizeof(Database_SetIndexKeys_Params_Data)");
const uint32_t kDatabase_SetIndexesReady_Name = 12;
class Database_SetIndexesReady_Params_Data {
 public:
  static Database_SetIndexesReady_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_SetIndexesReady_Params_Data))) Database_SetIndexesReady_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<int64_t>> index_ids;

 private:
  Database_SetIndexesReady_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_SetIndexesReady_Params_Data() = delete;
};
static_assert(sizeof(Database_SetIndexesReady_Params_Data) == 32,
              "Bad sizeof(Database_SetIndexesReady_Params_Data)");
const uint32_t kDatabase_OpenCursor_Name = 13;
class Database_OpenCursor_Params_Data {
 public:
  static Database_OpenCursor_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_OpenCursor_Params_Data))) Database_OpenCursor_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<internal::KeyRange_Data> key_range;
  int32_t direction;
  uint8_t key_only : 1;
  uint8_t pad5_[3];
  int32_t task_type;
  mojo::internal::AssociatedInterface_Data callbacks;
  uint8_t padfinal_[4];

 private:
  Database_OpenCursor_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_OpenCursor_Params_Data() = delete;
};
static_assert(sizeof(Database_OpenCursor_Params_Data) == 64,
              "Bad sizeof(Database_OpenCursor_Params_Data)");
const uint32_t kDatabase_Count_Name = 14;
class Database_Count_Params_Data {
 public:
  static Database_Count_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Count_Params_Data))) Database_Count_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<internal::KeyRange_Data> key_range;
  mojo::internal::AssociatedInterface_Data callbacks;

 private:
  Database_Count_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Count_Params_Data() = delete;
};
static_assert(sizeof(Database_Count_Params_Data) == 48,
              "Bad sizeof(Database_Count_Params_Data)");
const uint32_t kDatabase_DeleteRange_Name = 15;
class Database_DeleteRange_Params_Data {
 public:
  static Database_DeleteRange_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_DeleteRange_Params_Data))) Database_DeleteRange_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::Pointer<internal::KeyRange_Data> key_range;
  mojo::internal::AssociatedInterface_Data callbacks;

 private:
  Database_DeleteRange_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_DeleteRange_Params_Data() = delete;
};
static_assert(sizeof(Database_DeleteRange_Params_Data) == 40,
              "Bad sizeof(Database_DeleteRange_Params_Data)");
const uint32_t kDatabase_Clear_Name = 16;
class Database_Clear_Params_Data {
 public:
  static Database_Clear_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Clear_Params_Data))) Database_Clear_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  mojo::internal::AssociatedInterface_Data callbacks;

 private:
  Database_Clear_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Clear_Params_Data() = delete;
};
static_assert(sizeof(Database_Clear_Params_Data) == 32,
              "Bad sizeof(Database_Clear_Params_Data)");
const uint32_t kDatabase_CreateIndex_Name = 17;
class Database_CreateIndex_Params_Data {
 public:
  static Database_CreateIndex_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_CreateIndex_Params_Data))) Database_CreateIndex_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  mojo::internal::Pointer<internal::KeyPath_Data> key_path;
  uint8_t unique : 1;
  uint8_t multi_entry : 1;
  uint8_t padfinal_[7];

 private:
  Database_CreateIndex_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_CreateIndex_Params_Data() = delete;
};
static_assert(sizeof(Database_CreateIndex_Params_Data) == 56,
              "Bad sizeof(Database_CreateIndex_Params_Data)");
const uint32_t kDatabase_DeleteIndex_Name = 18;
class Database_DeleteIndex_Params_Data {
 public:
  static Database_DeleteIndex_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_DeleteIndex_Params_Data))) Database_DeleteIndex_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;

 private:
  Database_DeleteIndex_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_DeleteIndex_Params_Data() = delete;
};
static_assert(sizeof(Database_DeleteIndex_Params_Data) == 32,
              "Bad sizeof(Database_DeleteIndex_Params_Data)");
const uint32_t kDatabase_RenameIndex_Name = 19;
class Database_RenameIndex_Params_Data {
 public:
  static Database_RenameIndex_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_RenameIndex_Params_Data))) Database_RenameIndex_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;
  int64_t object_store_id;
  int64_t index_id;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> new_name;

 private:
  Database_RenameIndex_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_RenameIndex_Params_Data() = delete;
};
static_assert(sizeof(Database_RenameIndex_Params_Data) == 40,
              "Bad sizeof(Database_RenameIndex_Params_Data)");
const uint32_t kDatabase_Abort_Name = 20;
class Database_Abort_Params_Data {
 public:
  static Database_Abort_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Abort_Params_Data))) Database_Abort_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;

 private:
  Database_Abort_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Abort_Params_Data() = delete;
};
static_assert(sizeof(Database_Abort_Params_Data) == 16,
              "Bad sizeof(Database_Abort_Params_Data)");
const uint32_t kDatabase_Commit_Name = 21;
class Database_Commit_Params_Data {
 public:
  static Database_Commit_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_Commit_Params_Data))) Database_Commit_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t transaction_id;

 private:
  Database_Commit_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_Commit_Params_Data() = delete;
};
static_assert(sizeof(Database_Commit_Params_Data) == 16,
              "Bad sizeof(Database_Commit_Params_Data)");
const uint32_t kDatabase_AckReceivedBlobs_Name = 22;
class Database_AckReceivedBlobs_Params_Data {
 public:
  static Database_AckReceivedBlobs_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Database_AckReceivedBlobs_Params_Data))) Database_AckReceivedBlobs_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<mojo::internal::String_Data>>> uuids;

 private:
  Database_AckReceivedBlobs_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Database_AckReceivedBlobs_Params_Data() = delete;
};
static_assert(sizeof(Database_AckReceivedBlobs_Params_Data) == 16,
              "Bad sizeof(Database_AckReceivedBlobs_Params_Data)");
const uint32_t kFactory_GetDatabaseNames_Name = 0;
class Factory_GetDatabaseNames_Params_Data {
 public:
  static Factory_GetDatabaseNames_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Factory_GetDatabaseNames_Params_Data))) Factory_GetDatabaseNames_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::AssociatedInterface_Data callbacks;
  mojo::internal::Pointer<::url::mojom::internal::Origin_Data> origin;

 private:
  Factory_GetDatabaseNames_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Factory_GetDatabaseNames_Params_Data() = delete;
};
static_assert(sizeof(Factory_GetDatabaseNames_Params_Data) == 24,
              "Bad sizeof(Factory_GetDatabaseNames_Params_Data)");
const uint32_t kFactory_Open_Name = 1;
class Factory_Open_Params_Data {
 public:
  static Factory_Open_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Factory_Open_Params_Data))) Factory_Open_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t worker_thread;
  mojo::internal::AssociatedInterface_Data callbacks;
  mojo::internal::AssociatedInterface_Data database_callbacks;
  uint8_t pad2_[4];
  mojo::internal::Pointer<::url::mojom::internal::Origin_Data> origin;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;
  int64_t version;
  int64_t transaction_id;

 private:
  Factory_Open_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Factory_Open_Params_Data() = delete;
};
static_assert(sizeof(Factory_Open_Params_Data) == 64,
              "Bad sizeof(Factory_Open_Params_Data)");
const uint32_t kFactory_DeleteDatabase_Name = 2;
class Factory_DeleteDatabase_Params_Data {
 public:
  static Factory_DeleteDatabase_Params_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(Factory_DeleteDatabase_Params_Data))) Factory_DeleteDatabase_Params_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::AssociatedInterface_Data callbacks;
  mojo::internal::Pointer<::url::mojom::internal::Origin_Data> origin;
  mojo::internal::Pointer<::mojo::common::mojom::internal::String16_Data> name;

 private:
  Factory_DeleteDatabase_Params_Data() : header_({sizeof(*this), 0}) {
  }
  ~Factory_DeleteDatabase_Params_Data() = delete;
};
static_assert(sizeof(Factory_DeleteDatabase_Params_Data) == 32,
              "Bad sizeof(Factory_DeleteDatabase_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace indexed_db

#endif  // CONTENT_COMMON_INDEXED_DB_INDEXED_DB_MOJOM_SHARED_INTERNAL_H_