// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_FETCH_EVENT_DISPATCHER_MOJOM_SHARED_H_
#define CONTENT_COMMON_SERVICE_WORKER_FETCH_EVENT_DISPATCHER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "content/common/service_worker/fetch_event_dispatcher.mojom-shared-internal.h"
#include "content/common/url_loader.mojom-shared.h"
#include "mojo/common/common_custom_types.mojom-shared.h"
#include "third_party/WebKit/public/platform/modules/serviceworker/service_worker_event_status.mojom-shared.h"


namespace content {
namespace mojom {
using ServiceWorkerFetchRequestDataView = mojo::NativeStructDataView;

class FetchEventPreloadHandleDataView;



}  // namespace mojom
}  // namespace content

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::content::mojom::FetchEventPreloadHandleDataView> {
  using Data = ::content::mojom::internal::FetchEventPreloadHandle_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace content {
namespace mojom {
// Interface base classes. They are used for type safety check.
class FetchEventDispatcherInterfaceBase {};

using FetchEventDispatcherPtrDataView =
    mojo::InterfacePtrDataView<FetchEventDispatcherInterfaceBase>;
using FetchEventDispatcherRequestDataView =
    mojo::InterfaceRequestDataView<FetchEventDispatcherInterfaceBase>;
using FetchEventDispatcherAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FetchEventDispatcherInterfaceBase>;
using FetchEventDispatcherAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FetchEventDispatcherInterfaceBase>;
class FetchEventPreloadHandleDataView {
 public:
  FetchEventPreloadHandleDataView() {}

  FetchEventPreloadHandleDataView(
      internal::FetchEventPreloadHandle_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeUrlLoader() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::content::mojom::URLLoaderPtrDataView>(
            &data_->url_loader, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeUrlLoaderClientRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::content::mojom::URLLoaderClientRequestDataView>(
            &data_->url_loader_client_request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FetchEventPreloadHandle_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FetchEventDispatcher_DispatchFetchEvent_ParamsDataView {
 public:
  FetchEventDispatcher_DispatchFetchEvent_ParamsDataView() {}

  FetchEventDispatcher_DispatchFetchEvent_ParamsDataView(
      internal::FetchEventDispatcher_DispatchFetchEvent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t fetch_event_id() const {
    return data_->fetch_event_id;
  }
  inline void GetRequestDataView(
      ServiceWorkerFetchRequestDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequest(UserType* output) {
    auto* pointer = data_->request.Get();
    return mojo::internal::Deserialize<::content::mojom::ServiceWorkerFetchRequestDataView>(
        pointer, output, context_);
  }
  inline void GetPreloadHandleDataView(
      FetchEventPreloadHandleDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPreloadHandle(UserType* output) {
    auto* pointer = data_->preload_handle.Get();
    return mojo::internal::Deserialize<::content::mojom::FetchEventPreloadHandleDataView>(
        pointer, output, context_);
  }
 private:
  internal::FetchEventDispatcher_DispatchFetchEvent_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FetchEventDispatcher_DispatchFetchEvent_ResponseParamsDataView {
 public:
  FetchEventDispatcher_DispatchFetchEvent_ResponseParamsDataView() {}

  FetchEventDispatcher_DispatchFetchEvent_ResponseParamsDataView(
      internal::FetchEventDispatcher_DispatchFetchEvent_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::blink::mojom::ServiceWorkerEventStatus>(
        data_value, output);
  }

  ::blink::mojom::ServiceWorkerEventStatus status() const {
    return static_cast<::blink::mojom::ServiceWorkerEventStatus>(data_->status);
  }
  inline void GetDispatchEventTimeDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDispatchEventTime(UserType* output) {
    auto* pointer = data_->dispatch_event_time.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::FetchEventDispatcher_DispatchFetchEvent_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace content

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::content::mojom::FetchEventPreloadHandleDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::content::mojom::FetchEventPreloadHandleDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::content::mojom::internal::FetchEventPreloadHandle_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::content::mojom::internal::FetchEventPreloadHandle_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::content::mojom::internal::FetchEventPreloadHandle_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::url_loader, input, custom_context)) in_url_loader = CallWithContext(Traits::url_loader, input, custom_context);
    mojo::internal::Serialize<::content::mojom::URLLoaderPtrDataView>(
        in_url_loader, &result->url_loader, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->url_loader),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid url_loader in FetchEventPreloadHandle struct");
    decltype(CallWithContext(Traits::url_loader_client_request, input, custom_context)) in_url_loader_client_request = CallWithContext(Traits::url_loader_client_request, input, custom_context);
    mojo::internal::Serialize<::content::mojom::URLLoaderClientRequestDataView>(
        in_url_loader_client_request, &result->url_loader_client_request, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->url_loader_client_request),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid url_loader_client_request in FetchEventPreloadHandle struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::content::mojom::internal::FetchEventPreloadHandle_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::content::mojom::FetchEventPreloadHandleDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace content {
namespace mojom {



inline void FetchEventDispatcher_DispatchFetchEvent_ParamsDataView::GetRequestDataView(
    ServiceWorkerFetchRequestDataView* output) {
  auto pointer = data_->request.Get();
  *output = ServiceWorkerFetchRequestDataView(pointer, context_);
}
inline void FetchEventDispatcher_DispatchFetchEvent_ParamsDataView::GetPreloadHandleDataView(
    FetchEventPreloadHandleDataView* output) {
  auto pointer = data_->preload_handle.Get();
  *output = FetchEventPreloadHandleDataView(pointer, context_);
}


inline void FetchEventDispatcher_DispatchFetchEvent_ResponseParamsDataView::GetDispatchEventTimeDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->dispatch_event_time.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_SERVICE_WORKER_FETCH_EVENT_DISPATCHER_MOJOM_SHARED_H_
