// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_HYPHENATION_HYPHENATION_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_HYPHENATION_HYPHENATION_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/hyphenation/hyphenation.mojom-shared.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "ui/gfx/geometry/point.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "ui/gfx/geometry/insets.h"
#include "base/version.h"
#include "base/time/time.h"
#include "third_party/WebKit/public/web/WebWindowFeatures.h"
#include "base/strings/string16.h"
#include "content/public/common/media_metadata.h"
#include "content/common/bluetooth/web_bluetooth_device_id.h"
#include "ui/gfx/geometry/size.h"
#include "content/public/common/referrer.h"
#include "base/values.h"
#include "ui/gfx/geometry/rect_f.h"
#include "base/files/file_path.h"
#include "url/origin.h"
#include "ui/gfx/geometry/point_f.h"
#include "base/unguessable_token.h"
#include "third_party/WebKit/public/platform/WebReferrerPolicy.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "url/gurl.h"
#include "device/bluetooth/bluetooth_uuid.h"
#include "ui/gfx/geometry/rect.h"
#include "ui/gfx/geometry/vector2d.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class Hyphenation;
using HyphenationPtr = mojo::InterfacePtr<Hyphenation>;
using HyphenationPtrInfo = mojo::InterfacePtrInfo<Hyphenation>;
using ThreadSafeHyphenationPtr =
    mojo::ThreadSafeInterfacePtr<Hyphenation>;
using HyphenationRequest = mojo::InterfaceRequest<Hyphenation>;
using HyphenationAssociatedPtr =
    mojo::AssociatedInterfacePtr<Hyphenation>;
using ThreadSafeHyphenationAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Hyphenation>;
using HyphenationAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Hyphenation>;
using HyphenationAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Hyphenation>;


class HyphenationProxy;

template <typename ImplRefTraits>
class HyphenationStub;

class HyphenationRequestValidator;
class HyphenationResponseValidator;

class CONTENT_EXPORT Hyphenation
    : public HyphenationInterfaceBase {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = true;

  using Proxy_ = HyphenationProxy;

  template <typename ImplRefTraits>
  using Stub_ = HyphenationStub<ImplRefTraits>;

  using RequestValidator_ = HyphenationRequestValidator;
  using ResponseValidator_ = HyphenationResponseValidator;
  virtual ~Hyphenation() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool OpenDictionary(const mojo::String& locale, mojo::ScopedHandle* hyphenation_dictionary_handle);

  using OpenDictionaryCallback = base::Callback<void(mojo::ScopedHandle)>;
  virtual void OpenDictionary(const mojo::String& locale, const OpenDictionaryCallback& callback) = 0;
};

class CONTENT_EXPORT HyphenationProxy
    : public Hyphenation {
 public:
  explicit HyphenationProxy(mojo::MessageReceiverWithResponder* receiver);
  bool OpenDictionary(const mojo::String& locale, mojo::ScopedHandle* hyphenation_dictionary_handle) override;
  void OpenDictionary(const mojo::String& locale, const OpenDictionaryCallback& callback) override;

  void set_group_controller(
        scoped_refptr<mojo::AssociatedGroupController> group_controller) {
    group_controller_ = std::move(group_controller);
  }
 private:
  mojo::MessageReceiverWithResponder* receiver_;
  scoped_refptr<mojo::AssociatedGroupController> group_controller_;
};
class CONTENT_EXPORT HyphenationStubDispatch {
 public:
  static bool Accept(Hyphenation* impl,
                     mojo::internal::SerializationContext* context,
                     mojo::Message* message);
  static bool AcceptWithResponder(Hyphenation* impl,
                                  mojo::internal::SerializationContext* context,
                                  mojo::Message* message,
                                  mojo::MessageReceiverWithStatus* responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Hyphenation>>
class HyphenationStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  HyphenationStub() {}
  ~HyphenationStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HyphenationStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HyphenationStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message,
        responder);
  }

 private:
  ImplPointerType sink_;
  mojo::internal::SerializationContext serialization_context_;
};
class CONTENT_EXPORT HyphenationRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT HyphenationResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_HYPHENATION_HYPHENATION_MOJOM_H_