// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_SHARED_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/WebKit/public/platform/modules/shapedetection/shapedetection.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"


namespace blink {
namespace mojom {
class FaceDetectionResultDataView;

class BarcodeDetectionResultDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::FaceDetectionResultDataView> {
  using Data = ::blink::mojom::internal::FaceDetectionResult_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::BarcodeDetectionResultDataView> {
  using Data = ::blink::mojom::internal::BarcodeDetectionResult_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {
// Interface base classes. They are used for type safety check.
class ShapeDetectionInterfaceBase {};

using ShapeDetectionPtrDataView =
    mojo::InterfacePtrDataView<ShapeDetectionInterfaceBase>;
using ShapeDetectionRequestDataView =
    mojo::InterfaceRequestDataView<ShapeDetectionInterfaceBase>;
using ShapeDetectionAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ShapeDetectionInterfaceBase>;
using ShapeDetectionAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ShapeDetectionInterfaceBase>;
class FaceDetectionResultDataView {
 public:
  FaceDetectionResultDataView() {}

  FaceDetectionResultDataView(
      internal::FaceDetectionResult_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetBoundingboxesDataView(
      mojo::ArrayDataView<::gfx::mojom::RectFDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBoundingboxes(UserType* output) {
    auto* pointer = data_->boundingBoxes.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::gfx::mojom::RectFDataView>>(
        pointer, output, context_);
  }
 private:
  internal::FaceDetectionResult_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BarcodeDetectionResultDataView {
 public:
  BarcodeDetectionResultDataView() {}

  BarcodeDetectionResultDataView(
      internal::BarcodeDetectionResult_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRawValueDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRawValue(UserType* output) {
    auto* pointer = data_->raw_value.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetBoundingBoxDataView(
      ::gfx::mojom::RectFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBoundingBox(UserType* output) {
    auto* pointer = data_->bounding_box.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectFDataView>(
        pointer, output, context_);
  }
 private:
  internal::BarcodeDetectionResult_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ShapeDetection_DetectFaces_ParamsDataView {
 public:
  ShapeDetection_DetectFaces_ParamsDataView() {}

  ShapeDetection_DetectFaces_ParamsDataView(
      internal::ShapeDetection_DetectFaces_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedSharedBufferHandle TakeFrameData() {
    mojo::ScopedSharedBufferHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedSharedBufferHandle>(
            &data_->frame_data, &result, context_);
    DCHECK(ret);
    return result;
  }
  uint32_t width() const {
    return data_->width;
  }
  uint32_t height() const {
    return data_->height;
  }
 private:
  internal::ShapeDetection_DetectFaces_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ShapeDetection_DetectFaces_ResponseParamsDataView {
 public:
  ShapeDetection_DetectFaces_ResponseParamsDataView() {}

  ShapeDetection_DetectFaces_ResponseParamsDataView(
      internal::ShapeDetection_DetectFaces_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      FaceDetectionResultDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::blink::mojom::FaceDetectionResultDataView>(
        pointer, output, context_);
  }
 private:
  internal::ShapeDetection_DetectFaces_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ShapeDetection_DetectBarcodes_ParamsDataView {
 public:
  ShapeDetection_DetectBarcodes_ParamsDataView() {}

  ShapeDetection_DetectBarcodes_ParamsDataView(
      internal::ShapeDetection_DetectBarcodes_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedSharedBufferHandle TakeFrameData() {
    mojo::ScopedSharedBufferHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedSharedBufferHandle>(
            &data_->frame_data, &result, context_);
    DCHECK(ret);
    return result;
  }
  uint32_t width() const {
    return data_->width;
  }
  uint32_t height() const {
    return data_->height;
  }
 private:
  internal::ShapeDetection_DetectBarcodes_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ShapeDetection_DetectBarcodes_ResponseParamsDataView {
 public:
  ShapeDetection_DetectBarcodes_ResponseParamsDataView() {}

  ShapeDetection_DetectBarcodes_ResponseParamsDataView(
      internal::ShapeDetection_DetectBarcodes_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultsDataView(
      mojo::ArrayDataView<BarcodeDetectionResultDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResults(UserType* output) {
    auto* pointer = data_->results.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::BarcodeDetectionResultDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ShapeDetection_DetectBarcodes_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FaceDetectionResultDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::FaceDetectionResultDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::FaceDetectionResult_Data);
    decltype(CallWithContext(Traits::boundingBoxes, input, custom_context)) in_boundingBoxes = CallWithContext(Traits::boundingBoxes, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::gfx::mojom::RectFDataView>>(
        in_boundingBoxes, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::FaceDetectionResult_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::FaceDetectionResult_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::boundingBoxes, input, custom_context)) in_boundingBoxes = CallWithContext(Traits::boundingBoxes, input, custom_context);
    typename decltype(result->boundingBoxes)::BaseType* boundingBoxes_ptr;
    const mojo::internal::ContainerValidateParams boundingBoxes_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::gfx::mojom::RectFDataView>>(
        in_boundingBoxes, buffer, &boundingBoxes_ptr, &boundingBoxes_validate_params,
        context);
    result->boundingBoxes.Set(boundingBoxes_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->boundingBoxes.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null boundingBoxes in FaceDetectionResult struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::FaceDetectionResult_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::FaceDetectionResultDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::BarcodeDetectionResultDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::BarcodeDetectionResultDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::BarcodeDetectionResult_Data);
    decltype(CallWithContext(Traits::raw_value, input, custom_context)) in_raw_value = CallWithContext(Traits::raw_value, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_raw_value, context);
    decltype(CallWithContext(Traits::bounding_box, input, custom_context)) in_bounding_box = CallWithContext(Traits::bounding_box, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectFDataView>(
        in_bounding_box, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::BarcodeDetectionResult_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::BarcodeDetectionResult_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::raw_value, input, custom_context)) in_raw_value = CallWithContext(Traits::raw_value, input, custom_context);
    typename decltype(result->raw_value)::BaseType* raw_value_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_raw_value, buffer, &raw_value_ptr, context);
    result->raw_value.Set(raw_value_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->raw_value.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null raw_value in BarcodeDetectionResult struct");
    decltype(CallWithContext(Traits::bounding_box, input, custom_context)) in_bounding_box = CallWithContext(Traits::bounding_box, input, custom_context);
    typename decltype(result->bounding_box)::BaseType* bounding_box_ptr;
    mojo::internal::Serialize<::gfx::mojom::RectFDataView>(
        in_bounding_box, buffer, &bounding_box_ptr, context);
    result->bounding_box.Set(bounding_box_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->bounding_box.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null bounding_box in BarcodeDetectionResult struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::BarcodeDetectionResult_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::BarcodeDetectionResultDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {

inline void FaceDetectionResultDataView::GetBoundingboxesDataView(
    mojo::ArrayDataView<::gfx::mojom::RectFDataView>* output) {
  auto pointer = data_->boundingBoxes.Get();
  *output = mojo::ArrayDataView<::gfx::mojom::RectFDataView>(pointer, context_);
}


inline void BarcodeDetectionResultDataView::GetRawValueDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->raw_value.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BarcodeDetectionResultDataView::GetBoundingBoxDataView(
    ::gfx::mojom::RectFDataView* output) {
  auto pointer = data_->bounding_box.Get();
  *output = ::gfx::mojom::RectFDataView(pointer, context_);
}




inline void ShapeDetection_DetectFaces_ResponseParamsDataView::GetResultDataView(
    FaceDetectionResultDataView* output) {
  auto pointer = data_->result.Get();
  *output = FaceDetectionResultDataView(pointer, context_);
}




inline void ShapeDetection_DetectBarcodes_ResponseParamsDataView::GetResultsDataView(
    mojo::ArrayDataView<BarcodeDetectionResultDataView>* output) {
  auto pointer = data_->results.Get();
  *output = mojo::ArrayDataView<BarcodeDetectionResultDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_SHARED_H_
