// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/shapedetection/shapedetection.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "base/values.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "base/strings/string16.h"
#include "base/version.h"
#include "base/time/time.h"
#include "third_party/WebKit/public/web/WebWindowFeatures.h"
#include "ui/gfx/geometry/insets.h"
#include "content/public/common/media_metadata.h"
#include "content/common/bluetooth/web_bluetooth_device_id.h"
#include "ui/gfx/geometry/size.h"
#include "content/public/common/referrer.h"
#include "ui/gfx/geometry/point.h"
#include "ui/gfx/geometry/rect_f.h"
#include "base/files/file_path.h"
#include "url/origin.h"
#include "ui/gfx/geometry/point_f.h"
#include "base/unguessable_token.h"
#include "third_party/WebKit/public/platform/WebReferrerPolicy.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "url/gurl.h"
#include "device/bluetooth/bluetooth_uuid.h"
#include "ui/gfx/geometry/rect.h"
#include "ui/gfx/geometry/vector2d.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class ShapeDetection;
using ShapeDetectionPtr = mojo::InterfacePtr<ShapeDetection>;
using ShapeDetectionPtrInfo = mojo::InterfacePtrInfo<ShapeDetection>;
using ThreadSafeShapeDetectionPtr =
    mojo::ThreadSafeInterfacePtr<ShapeDetection>;
using ShapeDetectionRequest = mojo::InterfaceRequest<ShapeDetection>;
using ShapeDetectionAssociatedPtr =
    mojo::AssociatedInterfacePtr<ShapeDetection>;
using ThreadSafeShapeDetectionAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ShapeDetection>;
using ShapeDetectionAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ShapeDetection>;
using ShapeDetectionAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ShapeDetection>;

class FaceDetectionResult;
using FaceDetectionResultPtr = mojo::StructPtr<FaceDetectionResult>;

class BarcodeDetectionResult;
using BarcodeDetectionResultPtr = mojo::StructPtr<BarcodeDetectionResult>;


class ShapeDetectionProxy;

template <typename ImplRefTraits>
class ShapeDetectionStub;

class ShapeDetectionRequestValidator;
class ShapeDetectionResponseValidator;

class CONTENT_EXPORT ShapeDetection
    : public ShapeDetectionInterfaceBase {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = ShapeDetectionProxy;

  template <typename ImplRefTraits>
  using Stub_ = ShapeDetectionStub<ImplRefTraits>;

  using RequestValidator_ = ShapeDetectionRequestValidator;
  using ResponseValidator_ = ShapeDetectionResponseValidator;
  virtual ~ShapeDetection() {}


  using DetectFacesCallback = base::Callback<void(FaceDetectionResultPtr)>;
  virtual void DetectFaces(mojo::ScopedSharedBufferHandle frame_data, uint32_t width, uint32_t height, const DetectFacesCallback& callback) = 0;


  using DetectBarcodesCallback = base::Callback<void(std::vector<BarcodeDetectionResultPtr>)>;
  virtual void DetectBarcodes(mojo::ScopedSharedBufferHandle frame_data, uint32_t width, uint32_t height, const DetectBarcodesCallback& callback) = 0;
};

class CONTENT_EXPORT ShapeDetectionProxy
    : public ShapeDetection {
 public:
  explicit ShapeDetectionProxy(mojo::MessageReceiverWithResponder* receiver);
  void DetectFaces(mojo::ScopedSharedBufferHandle frame_data, uint32_t width, uint32_t height, const DetectFacesCallback& callback) override;
  void DetectBarcodes(mojo::ScopedSharedBufferHandle frame_data, uint32_t width, uint32_t height, const DetectBarcodesCallback& callback) override;

  void set_group_controller(
        scoped_refptr<mojo::AssociatedGroupController> group_controller) {
    group_controller_ = std::move(group_controller);
  }
 private:
  mojo::MessageReceiverWithResponder* receiver_;
  scoped_refptr<mojo::AssociatedGroupController> group_controller_;
};
class CONTENT_EXPORT ShapeDetectionStubDispatch {
 public:
  static bool Accept(ShapeDetection* impl,
                     mojo::internal::SerializationContext* context,
                     mojo::Message* message);
  static bool AcceptWithResponder(ShapeDetection* impl,
                                  mojo::internal::SerializationContext* context,
                                  mojo::Message* message,
                                  mojo::MessageReceiverWithStatus* responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ShapeDetection>>
class ShapeDetectionStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ShapeDetectionStub() {}
  ~ShapeDetectionStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ShapeDetectionStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ShapeDetectionStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message,
        responder);
  }

 private:
  ImplPointerType sink_;
  mojo::internal::SerializationContext serialization_context_;
};
class CONTENT_EXPORT ShapeDetectionRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT ShapeDetectionResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};






class CONTENT_EXPORT FaceDetectionResult {
 public:
  using DataView = FaceDetectionResultDataView;
  using Data_ = internal::FaceDetectionResult_Data;

  static FaceDetectionResultPtr New();

  template <typename U>
  static FaceDetectionResultPtr From(const U& u) {
    return mojo::TypeConverter<FaceDetectionResultPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, FaceDetectionResult>::Convert(*this);
  }

  FaceDetectionResult();
  ~FaceDetectionResult();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = FaceDetectionResultPtr>
  FaceDetectionResultPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, FaceDetectionResult>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        FaceDetectionResult::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        FaceDetectionResult::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  std::vector<gfx::RectF> boundingBoxes;
};



class CONTENT_EXPORT BarcodeDetectionResult {
 public:
  using DataView = BarcodeDetectionResultDataView;
  using Data_ = internal::BarcodeDetectionResult_Data;

  static BarcodeDetectionResultPtr New();

  template <typename U>
  static BarcodeDetectionResultPtr From(const U& u) {
    return mojo::TypeConverter<BarcodeDetectionResultPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BarcodeDetectionResult>::Convert(*this);
  }

  BarcodeDetectionResult();
  ~BarcodeDetectionResult();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BarcodeDetectionResultPtr>
  BarcodeDetectionResultPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BarcodeDetectionResult>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BarcodeDetectionResult::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BarcodeDetectionResult::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  std::string raw_value;
  gfx::RectF bounding_box;
};


template <typename StructPtrType>
FaceDetectionResultPtr FaceDetectionResult::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->boundingBoxes = mojo::internal::Clone(boundingBoxes);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, FaceDetectionResult>::value>::type*>
bool FaceDetectionResult::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->boundingBoxes, other.boundingBoxes))
    return false;
  return true;
}
template <typename StructPtrType>
BarcodeDetectionResultPtr BarcodeDetectionResult::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->raw_value = mojo::internal::Clone(raw_value);
  rv->bounding_box = mojo::internal::Clone(bounding_box);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BarcodeDetectionResult>::value>::type*>
bool BarcodeDetectionResult::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->raw_value, other.raw_value))
    return false;
  if (!mojo::internal::Equals(this->bounding_box, other.bounding_box))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::FaceDetectionResult::DataView,
                                         ::blink::mojom::FaceDetectionResultPtr> {
  static bool IsNull(const ::blink::mojom::FaceDetectionResultPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::FaceDetectionResultPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::FaceDetectionResult::boundingBoxes)& boundingBoxes(
      const ::blink::mojom::FaceDetectionResultPtr& input) {
    return input->boundingBoxes;
  }

  static bool Read(::blink::mojom::FaceDetectionResult::DataView input, ::blink::mojom::FaceDetectionResultPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::BarcodeDetectionResult::DataView,
                                         ::blink::mojom::BarcodeDetectionResultPtr> {
  static bool IsNull(const ::blink::mojom::BarcodeDetectionResultPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::BarcodeDetectionResultPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::BarcodeDetectionResult::raw_value)& raw_value(
      const ::blink::mojom::BarcodeDetectionResultPtr& input) {
    return input->raw_value;
  }

  static const decltype(::blink::mojom::BarcodeDetectionResult::bounding_box)& bounding_box(
      const ::blink::mojom::BarcodeDetectionResultPtr& input) {
    return input->bounding_box;
  }

  static bool Read(::blink::mojom::BarcodeDetectionResult::DataView input, ::blink::mojom::BarcodeDetectionResultPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_SHAPEDETECTION_SHAPEDETECTION_MOJOM_H_